import { useState, useCallback, useEffect } from 'react';

const createStoreHook = store => key => {
  const [value, setValue] = useState(store.get(key));
  const setter = useCallback(newValue => store.set(key, newValue), [key]);
  useEffect(() => {
    const subscription = store.subscribe(key, setValue, {
      withInitialValue: true
    });
    return () => subscription.unsubscribe();
  }, []);
  return [value, setter];
};

export { createStoreHook };
